<?php

/**
 * @file
 * Forms for creating / editing returns.
 */

/**
 * Form callback: edit the global return settings.
 *
 * @param array $form
 *   The node being added or edited.
 * @param array $form_state
 *   The form state array.
 *
 * @return string
 *   Form to render.
 */
function commerce_rma_settings_form($form, &$form_state) {
  $form['commerce_rma_return_is_activated'] = array(
    '#type' => 'checkbox',
    '#title' => t('Activate returns'),
    '#default_value' => variable_get('commerce_rma_return_is_activated', TRUE),
    '#return_value' => TRUE,
  );

  $form['commerce_rma_return_period'] = array(
    '#type' => 'fieldset',
    '#title' => t('Return period'),
    '#attributes' => array('class' => array('container-inline')),
    '#tree' => TRUE,
    '#states' => array(
      'invisible' => array(
        'input[name="commerce_rma_return_is_activated"]' => array('checked' => FALSE),
      ),
    ),
  );

  $return_period = variable_get('commerce_rma_return_period', array('unit' => 0, 'granularity' => 'D'));

  $form['commerce_rma_return_period']['unit'] = array(
    '#type' => 'textfield',
    '#size' => 3,
    '#default_value' => $return_period['unit'],
  );

  $form['commerce_rma_return_period']['granularity'] = array(
    '#type' => 'select',
    '#options' =>  array('D' => t('days'), 'M' => t('months'), 'Y' => t('years')),
    '#default_value' => $return_period['granularity'],
    '#description' => t('define a time interval where the customer is allowed to return the products of the order.'),
  );

  $form['commerce_rma_advanced_settings'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#title' => t('Advanced settings'),
    '#tree' => TRUE,
    '#states' => array(
      'invisible' => array(
        'input[name="commerce_rma_return_is_activated"]' => array('checked' => FALSE),
      ),
    ),
  );

  $return_settings = variable_get('commerce_rma_advanced_settings', array('order_status' => 'completed'));

  $form['commerce_rma_advanced_settings']['order_status'] = array(
    '#type' => 'select',
    '#title' => t('Order status'),
    '#options' => commerce_order_state_options_list(),
    '#default_value' => $return_settings['order_status'],
    '#description' => t('Select the order status which allows a user to create a return.'),
  );

  return system_settings_form($form);
}

/**
 * Form callback: create or edit a return.
 *
 * @param array $form
 *   The node being added or edited.
 * @param array $form_state
 *   The form state array.
 * @param object $commerce_order
 *   The order object to edit through the form.
 * @param object $commerce_order_return
 *   The return object to edit through the form.
 *
 * @return array
 *   The form to render.
 */
function commerce_rma_return_form($form, &$form_state, $commerce_order = NULL, $commerce_order_return = NULL) {
  // Ensure this include file is loaded when the form is rebuilt from the cache.
  $form_state['build_info']['files']['form'] = drupal_get_path('module', 'commerce_rma') . '/includes/commerce_rma.forms.inc';
  $form_state['#attached']['css'][] = drupal_get_path('module', 'commerce_rma') . '/theme/commerce_rma.theme.css';

  if (!is_null($commerce_order)) {
    // Create a new return object from order.
    $commerce_order_return = commerce_rma_commerce_return_new($commerce_order);
  }

  // Ensure the owner name is accessible if the uid is set.
  if (!empty($commerce_order_return->uid) && $owner = user_load($commerce_order_return->uid)) {
    $commerce_order_return->name = $owner->name;

    if (empty($commerce_order_return->mail)) {
      $commerce_order_return->mail = $owner->mail;
    }
  }

  // If return is new.
  if (isset($commerce_order_return->is_new)) {
    $commerce_order_return->status = 'return_processing';
  }

  if (empty($commerce_order_return->created)) {
    $commerce_order_return->date = format_date(REQUEST_TIME, 'custom', 'Y-m-d H:i:s O');
  }

  // Add the field related form elements.
  $form_state['commerce_order_return'] = $commerce_order_return;
  field_attach_form('commerce_order', $commerce_order_return, $form, $form_state);

  // Hide the return total and order reference fields from direct editing.
  $form['commerce_order_total']['#access'] = FALSE;
  $form['commerce_order_reference']['#access'] = FALSE;

  $form['additional_settings'] = array(
    '#type' => 'vertical_tabs',
    '#weight' => 99,
  );

  // Build an array of return status options grouped by return state.
  $options = array();

  foreach (commerce_rma_commerce_return_state_info() as $state_name => $state) {
    foreach (commerce_rma_commerce_return_status_info() as $status_name => $status) {
      if ($status['state'] == $state_name) {
        $options[$state['title']][$status_name] = check_plain($status['title']);
      }
    }
  }

  // Add a section to update the status and leave a log message.
  $form['return_status'] = array(
    '#type' => 'fieldset',
    '#title' => t('Return status'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#group' => 'additional_settings',
    '#attached' => array(
      'js' => array(
        drupal_get_path('module', 'commerce_order') . '/commerce_order.js',
        array(
          'type' => 'setting',
          'data' => array('status_titles' => commerce_order_status_get_title()),
        ),
      ),
    ),
    '#weight' => 20,
  );
  $form['return_status']['status'] = array(
    '#type' => 'select',
    '#title' => t('Status'),
    '#options' => $options,
    '#default_value' => $commerce_order_return->status,
  );
  $form['return_status']['status_original'] = array(
    '#type' => 'hidden',
    '#value' => $commerce_order_return->status,
    '#attributes' => array('id' => 'edit-status-original'),
  );
  $form['return_status']['log'] = array(
    '#type' => 'textarea',
    '#title' => t('Update log message'),
    '#description' => t('Provide an explanation of the changes you are making. This will provide a meaningful audit trail for updates to this return.'),
    '#rows' => 4,
  );

  // Add the user account and e-mail fields.
  $form['user'] = array(
    '#type' => 'fieldset',
    '#title' => t('User information'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#access' => user_access('administer commerce_order entities'),
    '#group' => 'additional_settings',
    '#attached' => array(
      'js' => array(
        drupal_get_path('module', 'commerce_order') . '/commerce_order.js',
        array(
          'type' => 'setting',
          'data' => array('anonymous' => variable_get('anonymous', t('Anonymous'))),
        ),
      ),
    ),
    '#weight' => 30,
  );
  $form['user']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Owned by'),
    '#description' => t('Leave blank for %anonymous.', array('%anonymous' => variable_get('anonymous', t('Anonymous')))),
    '#maxlength' => 60,
    '#autocomplete_path' => 'user/autocomplete',
    '#default_value' => !empty($commerce_order_return->name) ? $commerce_order_return->name : '',
    '#weight' => -1,
  );
  $form['user']['mail'] = array(
    '#type' => 'textfield',
    '#title' => t('Return contact e-mail'),
    '#description' => t('Defaults to the owner account e-mail address if left blank. Used for return communication.'),
    '#default_value' => $commerce_order_return->mail,
  );

  // Add a log checkbox and timestamp field to a history tab.
  $form['return_history'] = array(
    '#type' => 'fieldset',
    '#title' => t('Return history', array(), array('context' => 'a drupal commerce return')),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#group' => 'additional_settings',
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'commerce_order') . '/commerce_order.js'),
    ),
    '#weight' => 40,
  );
  $form['return_history']['revision'] = array(
    '#type' => 'checkbox',
    '#title' => t('Create new revision on update'),
    '#description' => t('If an update log message is entered, a revision will be created even if this is unchecked.'),
    '#default_value' => variable_get('commerce_order_auto_revision', TRUE),
    '#access' => user_access('administer commerce_order entities'),
  );
  $form['return_history']['date'] = array(
    '#type' => 'textfield',
    '#title' => t('Created on'),
    '#description' => t(
      'Format: %time. The date format is YYYY-MM-DD and %timezone is the time zone offset from UTC. Leave blank to use the time of form submission.', array(
        '%time' => !empty($commerce_order_return->date) ? date_format(date_create($commerce_order_return->date), 'Y-m-d H:i:s O') : format_date($commerce_order_return->created, 'custom', 'Y-m-d H:i:s O'),
        '%timezone' => !empty($commerce_order_return->date) ? date_format(date_create($commerce_order_return->date), 'O') : format_date($commerce_order_return->created, 'custom', 'O'),
      )
    ),
    '#maxlength' => 25,
    '#default_value' => !empty($commerce_order_return->created) ? format_date($commerce_order_return->created, 'custom', 'Y-m-d H:i:s O') : '',
  );
  $form['return_history']['created'] = array(
    '#type' => 'hidden',
    '#value' => !empty($commerce_order_return->created) ? format_date($commerce_order_return->created, 'short') : '',
    '#attributes' => array('id' => 'edit-created'),
  );
  $form['return_history']['changed'] = array(
    '#type' => 'hidden',
    '#value' => !empty($commerce_order_return->changed) ? format_date($commerce_order_return->changed, 'short') : '',
    '#attributes' => array('id' => 'edit-changed'),
  );

  $form['actions'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('form-return-actions')),
    '#weight' => 100,
  );

  // Set the default_value of commerce_order_reference to the good one.
  if (isset($commerce_order_return->target_id)) {
    $form['commerce_order_reference'][LANGUAGE_NONE]['#default_value'] = $commerce_order_return->target_id;
  }

  // We add the form's #submit array to this button along with the actual submit
  // handler to preserve any submit handlers added by a form callback_wrapper.
  $submit = array();

  if (!empty($form['#submit'])) {
    $submit += $form['#submit'];
  }

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save return', array(), array('context' => 'a drupal commerce return')),
    '#submit' => array_merge($submit, array('commerce_rma_return_form_submit')),
    '#weight' => 40,
  );

  // We append the validate handler to #validate in case a form callback_wrapper
  // is used to add validate handlers earlier.
  $form['#validate'][] = 'commerce_rma_return_form_validate';

  return $form;
}

/**
 * Validation callback for commerce_rma_return_form().
 *
 * @param array $form
 *   The node being added or edited.
 * @param array $form_state
 *   The form state array.
 */
function commerce_rma_return_form_validate($form, &$form_state) {
  $commerce_order_return = $form_state['commerce_order_return'];

  // Validate the "owned by" field.
  if (!empty($form_state['values']['name']) && !($account = user_load_by_name($form_state['values']['name']))) {
    // The use of empty() is mandatory in the context of usernames as the empty
    // string denotes an anonymous user.
    form_set_error('name', t('The username %name does not exist.', array('%name' => $form_state['values']['name'])));
  }

  // Validate the "created on" field.
  if (!empty($form_state['values']['date']) && strtotime($form_state['values']['date']) === FALSE) {
    form_set_error('date', t('You have to specify a valid date.'));
  }

  // Validate the e-mail address entered.
  if (!empty($form_state['values']['mail']) && !valid_email_address($form_state['values']['mail'])) {
    form_set_error('mail', t('You have specified an invalid e-mail address.'));
  }

  // Notify field widgets to validate their data.
  field_attach_form_validate('commerce_order', $commerce_order_return, $form, $form_state);
}

/**
 * Submit callback for commerce_rma_return_form().
 *
 * @param array $form
 *   The node being added or edited.
 * @param array $form_state
 *   The form state array.
 */
function commerce_rma_return_form_submit($form, &$form_state) {

  // If the user is editing a return, load a fresh copy to merge changes to.
  if ($form_state['commerce_order_return']->order_id) {
    $form_state['commerce_order_return'] = commerce_order_load($form_state['commerce_order_return']->order_id);
  }

  // Merge changes into the commerce_order_return object in the form state so
  // it is accessible by field handlers.
  $commerce_order_return = $form_state['commerce_order_return'];

  if ($form_state['values']['revision'] || !empty($form_state['values']['log'])) {
    $commerce_order_return->revision = TRUE;
    $commerce_order_return->log = $form_state['values']['log'];
  }

  // Set the return's owner uid based on the supplied name.
  $converted = FALSE;
  $is_new = isset($commerce_order_return->is_new);

  if (!empty($form_state['values']['name']) && $account = user_load_by_name($form_state['values']['name'])) {
    // If the return is being converted to an authenticated return from an
    // anonymous return...
    if ($commerce_order_return->uid == 0) {
      // Set the converted boolean for later processing.
      $converted = TRUE;
    }

    $commerce_order_return->uid = $account->uid;

    if (empty($form_state['values']['mail'])) {
      $commerce_order_return->mail = $account->mail;
    }
  }
  else {
    $commerce_order_return->uid = 0;
  }

  if (!empty($form_state['values']['mail'])) {
    $commerce_order_return->mail = $form_state['values']['mail'];
  }

  $commerce_order_return->created = !empty($form_state['values']['date']) ? strtotime($form_state['values']['date']) : REQUEST_TIME;

  // Notify field widgets.
  field_attach_submit('commerce_order', $commerce_order_return, $form, $form_state);

  // Ensure the attached customer profile is associated with the order owner
  // if it does not have a uid yet and the order does.
  if ($converted) {
    $wrapper = entity_metadata_wrapper('commerce_order', $commerce_order_return);

    if (!is_null($wrapper->commerce_customer_rma->value()) && $wrapper->commerce_customer_rma->uid->value() == 0) {
      $wrapper->commerce_customer_rma->uid = $commerce_order_return->uid;
      $wrapper->commerce_customer_rma->save();
    }
  }

  // Update the return status if specified.
  if ($form_state['values']['status'] != $form_state['values']['status_original']) {
    // We skip return saving in the update since we do it below once for the
    // entire form submission.
    commerce_rma_status_update($commerce_order_return, $form_state['values']['status'], TRUE);
  }

  if ($is_new) {
    // Set the commerce_order_reference value.
    $wrapped_return = entity_metadata_wrapper('commerce_order', $commerce_order_return);
    $wrapped_return->commerce_order_reference = $commerce_order_return->target_id;
  }

  // Save the return.
  if (commerce_order_save($commerce_order_return)) {
    if ($is_new) {

      // Invoke the related rule event.
      rules_invoke_all('commerce_order_return_is_created', $commerce_order_return);

      $form_state['redirect'] = '/admin/commerce/returns/' . $commerce_order_return->order_id . '/view';

      drupal_set_message(
        t('Return number @number has been created.', array('@number' => $commerce_order_return->order_number))
      );
    }
    else {
      drupal_set_message(
        t('Return number @number has been successfully updated.', array('@number' => $commerce_order_return->order_number))
      );
    }

    // Ensure the attached line items are associated with the return if they do
    // not have an order_id set yet.
    foreach (entity_metadata_wrapper('commerce_order', $commerce_order_return)->commerce_line_items as $line_item_wrapper) {
      if ($line_item_wrapper->order_id->value() == 0) {
        $line_item_wrapper->order_id = $commerce_order_return->order_id;
        $line_item_wrapper->save();
      }
    }
  }
}

/**
 * Form callback: confirmation form for deleting a return.
 *
 * @param array $form
 *   The node being added or edited.
 * @param array $form_state
 *   The form state array.
 * @param object $commerce_order_return
 *   The return object to delete through the form.
 *
 * @return array
 *   The form array to remove a return.
 *
 * @see confirm_form()
 */
function commerce_rma_return_delete_form($form, &$form_state, $commerce_order_return) {
  $form_state['commerce_order_return'] = $commerce_order_return;

  // Ensure this include file is loaded when the form is rebuilt from the cache.
  $form_state['build_info']['files']['form'] = drupal_get_path('module', 'commerce_rma') . '/includes/commerce_rma.forms.inc';

  $form['#submit'][] = 'commerce_rma_return_delete_form_submit';

  $form = confirm_form(
    $form,
    t('Are you sure you want to delete return @number?', array('@number' => $commerce_order_return->order_number)),
    'admin/commerce/returns',
    '<p>' . t('Deleting this return cannot be undone.') . '</p>',
    t('Delete'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Submit callback for commerce_rma_return_delete_form().
 *
 * @param array $form
 *   The node being added or edited.
 * @param array $form_state
 *   The form state array.
 */
function commerce_rma_return_delete_form_submit($form, &$form_state) {
  $commerce_order_return = $form_state['commerce_order_return'];

  if (commerce_order_delete($commerce_order_return->order_id)) {
    drupal_set_message(t('Return @number has been deleted.', array('@number' => $commerce_order_return->order_number)));
    watchdog('commerce_return', 'Deleted return @number.', array('@number' => $commerce_order_return->order_number), WATCHDOG_NOTICE);
  }
  else {
    drupal_set_message(t('Return @number could not be deleted.', array('@number' => $commerce_order_return->order_number)), 'error');
  }

  // Redirect to admin returns list.
  $form_state['redirect'] = 'admin/commerce/returns';
}
